<?php
/**
 * The core plugin class.
 *
 * This is used to define internationalization, dashboard-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @author     WPALLSUPPORT <info@wpallsupport.com>
 */
class WPAS_EDD_PayU_Latam {

    private static $instance = null;

    public $gateway_id = 'wpas_edd_payu_latam';

    protected $version = WPAS_EDD_PAYU_LATAM_VERSION;

    public static function instance() {

        if ( is_null( self::$instance ) ) {
            self::$instance = new self();
        }

        return self::$instance;

    }

    public function __construct() {
        $this->setup_globals();
        $this->includes();
        $this->setup_actions();
    }

    private function setup_globals() {
	    $this->version = WPAS_EDD_PAYU_LATAM_VERSION;
	    $this->plugin_file = WPAS_EDD_PAYU_LATAM_FILE;
    }

    private function includes() {

        if ( !is_admin() ) {
            return;
        }

        require_once( WPAS_EDD_PAYU_LATAM_PATH . '/admin/class-edd-payu-latam-admin.php' );
        $this->admin = new WPAS_EDD_PayU_Latam_Admin();

    }

    private function setup_actions() {

        add_action( 'plugins_loaded', array( $this, 'load_plugin_textdomain' ) );
        add_action( 'edd_payment_gateways', array( $this, 'add_gateway' ) );
        add_action( 'edd_wpas_edd_payu_latam_cc_form', '__return_false' );
        add_action( 'edd_gateway_wpas_edd_payu_latam', array( $this, 'process_payment') );
        add_action( 'edd_enabled_payment_gateways', array( $this, 'enabled_payment_gateways') );
        add_action( 'init', array( $this, 'payu_check_callback') );

        if ( !is_admin() ) {
            return;
        }

        register_activation_hook( WPAS_EDD_PAYU_LATAM_FILE, array( $this->admin, 'activate' ) );
        register_deactivation_hook( WPAS_EDD_PAYU_LATAM_FILE, array( $this->admin, 'deactivate' ) );

        add_filter( 'edd_settings_sections_gateways', array( $this->admin, 'register_gateway_section' ) );
        add_filter( 'edd_settings_gateways', array( $this->admin, 'gateway_settings' ) );
    }

    public function load_plugin_textdomain(){
        load_plugin_textdomain('wpas-edd-payu-latam', FALSE, WPAS_EDD_PAYU_LATAM_PATH . '/languages/');
    }

    public function add_gateway( $gateways ) {

        $default_gateway_info = array(
            $this->gateway_id => array(
                'admin_label'    => __( 'PayUmoney Latam', 'wpas-edd-payu-latam' ),
                'checkout_label' => __( 'PayUmoney Latam', 'wpas-edd-payu-latam' ),
                'supports'       => array(),
            ),
        );

        $default_gateway_info = apply_filters( 'wpas_edd_register_payu_latam_gateway', $default_gateway_info );
        $gateways = array_merge( $gateways, $default_gateway_info );

        return $gateways;
    }

    public function enabled_payment_gateways( $gateway_list ) {

        $supported_currency = apply_filters( 'wpas_edd_payu_latam_supported_currencies', array( 'USD','COP','MXN','ARS','PEN','BRL','CLP' ) );

        if( !in_array( edd_get_currency(), $supported_currency)) {
            unset($gateway_list['wpas_edd_payu_latam']);
        }

        return $gateway_list;
    }

    public function process_payment( $purchase_data ) {

        if( ! wp_verify_nonce( $purchase_data['gateway_nonce'], 'edd-gateway' ) ) {
            wp_die( __( 'Nonce verification has failed', 'easy-digital-downloads' ), __( 'Error', 'easy-digital-downloads' ), array( 'response' => 403 ) );
        }

        global $edd_options;

        $errors = edd_get_errors();

        if (!$errors) {

            $currency = edd_get_currency();

            $payment = array(
                'price' => $purchase_data['price'],
                'date' => $purchase_data['date'],
                'user_email' => $purchase_data['user_email'],
                'purchase_key' => $purchase_data['purchase_key'],
                'currency' => $currency,
                'downloads' => $purchase_data['downloads'],
                'cart_details' => $purchase_data['cart_details'],
                'user_info' => $purchase_data['user_info'],
                'status' => 'pending'
            );

            $paymentID = edd_insert_payment($payment);

            $url = 'https://checkout.payulatam.com/ppp-web-gateway-payu/';
            if( edd_is_test_mode() ) {
                $url = 'https://sandbox.checkout.payulatam.com/ppp-web-gateway-payu/';
            }

            $redirect_text  = __( 'Redirecting to PayUmoney Latam site, click on button if not redirected.', 'wpas-edd-payu-latam' );
            $redirect_text  = apply_filters( 'wpas_edd_payu_latam_redirect_text', $redirect_text, $purchase_data );

            $apikey = !empty( $edd_options['payu_latam_api_key'] ) ? $edd_options['payu_latam_api_key'] :'';
            $merchant_id = !empty( $edd_options['payu_latam_merchant_id'] ) ? $edd_options['payu_latam_merchant_id'] :'';
            $referenceCode = apply_filters( 'wpas_payu_latam_form_reference_code', $paymentID, $purchase_data );
            $amount = !empty( $purchase_data['price'] ) ? $purchase_data['price']: 0;
            $confirmationUrl = add_query_arg( array( 'payment-confirm' => 'payu_latam', 'payu_latam_cb'=> 1, 'order-id' => $paymentID ), get_permalink( $edd_options['success_page'] ) );
            $responseUrl = add_query_arg( array( 'payment-confirm' => 'payu_latam', 'payu_latam_cb'=> 1, 'order-id' => $paymentID ), get_permalink( $edd_options['success_page'] ) );

            $signature = md5($apikey.'~'.$merchant_id.'~'.$referenceCode.'~'.$amount.'~'.$currency);

	        $account_id = !empty( $edd_options['payu_latam_account_id'] ) ? $edd_options['payu_latam_account_id'] :'';
	        $first_name = !empty( $purchase_data['user_info']['first_name'] ) ? $purchase_data['user_info']['first_name'] :'';
	        $last_name = !empty( $purchase_data['user_info']['last_name'] ) ? $purchase_data['user_info']['last_name'] :'';
	        $email = !empty( $purchase_data['user_email'] ) ? $purchase_data['user_email'] :'';

            $params                        = array();
            $params['merchantId']          = $merchant_id;
            $params['accountId']           = $account_id;
            $params['signature']           = $signature;
            $params['referenceCode']       = $referenceCode;
            $params['description']         = $paymentID;
            $params['amount']              = $amount;
            $params['tax']                 = $purchase_data['tax'];
            $params['taxReturnBase']       = 0;
            $params['currency']            = $currency;
            $params['test']                = edd_is_test_mode();
            $params['buyerFullName']       = trim($first_name. ' '. $last_name);
            $params['buyerEmail']          = $email;
            $params['telephone']           = '';
            $params['confirmationUrl']     = $confirmationUrl;
            $params['responseUrl']         = $responseUrl;

            $params = apply_filters( 'wpas_edd_payu_latam_form_extra_parameters', $params, $purchase_data );

            ?>
            <div class="wpas-edd-payu-latam-form" style="padding:20px;font-family:arial,sans-serif;text-align:center;color:#555">
                <h3><?php echo $redirect_text ;?></h3>
                <form name="wpas_edd_payu_latam_form" method="post" action="<?php echo $url; ?>">
                    <?php foreach ( $params as $param => $value ) { ?>
                        <input type="hidden" value="<?php echo esc_attr( $value );?>" name="<?php echo esc_attr( $param );?>">
                    <?php } ?>
                    <input type="submit" name="wpas_edd_payu_latam_submit" value="<?php esc_attr_e( 'Pay via PayUmoney Latam', 'wpas-edd-payu-latam' ) ; ?>">
                </form>
                <div class="loader"><img width="100px;" src="<?php echo WPAS_EDD_PAYU_LATAM_PLUGIN_URL.'assets/loading_spinner.gif'; ?>" alt="loader"></div>
                <script type="text/javascript">document.wpas_edd_payu_latam_form.submit();</script>
            </div>
            <?php

        }else{

            edd_record_gateway_error( __( 'Payment Error', 'easy-digital-downloads' ), sprintf( __( 'Payment creation failed while processing a PayUmoney payment gateway purchase. Payment data: %s', 'wpas-edd-payu-latam' ), json_encode( $purchase_data ) ) );
            // if errors are present, send the user back to the purchase page so they can be corrected
            edd_send_back_to_checkout('?payment-mode=' . $purchase_data['post_data']['edd-gateway']);

        }

    }

    public function payu_check_callback() {
        @ob_clean();
        global $edd_options;

        if ( isset( $_GET[ 'payu_latam_cb' ] ) && 1 == absint( $_GET[ 'payu_latam_cb' ] ) ) {
            $payment_id = absint($_REQUEST['description']);
            $txnid = !empty($_REQUEST['transactionId']) ? sanitize_text_field($_REQUEST['transactionId']) : '';

            $apikey = !empty( $edd_options['payu_latam_api_key'] ) ? $edd_options['payu_latam_api_key'] : '';
            $merchant_id = !empty($_REQUEST['merchantId']) ? sanitize_text_field($_REQUEST['merchantId']) : '';
            $referenceCode = !empty($_REQUEST['referenceCode']) ? sanitize_text_field($_REQUEST['referenceCode']) : '';

            $TX_VALUE = !empty($_REQUEST['TX_VALUE']) ? sanitize_text_field($_REQUEST['TX_VALUE']) : '';
            $New_value = number_format($TX_VALUE, 1, '.', '');

            $currency = sanitize_text_field($_REQUEST['currency']);

            $signature = !empty($_REQUEST['signature']) ? sanitize_text_field($_REQUEST['signature']) : '';
            $transactionState = !empty($_REQUEST['transactionState']) ? sanitize_text_field($_REQUEST['transactionState']) : '';

            $chain_signature = "$apikey~$merchant_id~$referenceCode~$New_value~$currency~$transactionState";
            $chain_signature = md5($chain_signature);


            if(strtoupper($signature) == strtoupper($chain_signature) && $payment_id > 0) {
                if ($transactionState == 4) {
                    // Approved
                    edd_update_payment_status( $payment_id, 'publish' );
                    edd_insert_payment_note( $payment_id, sprintf(__('Payment done via PayUmoney Latam with transaction id %s', 'wpas-edd-payu-latam'), $txnid));

                } else if ($transactionState == 6) {
                    // Rejected
                    edd_update_payment_status( $payment_id, 'failed' );
                    edd_insert_payment_note( $payment_id, sprintf(__('Payment done via PayUmoney Latam with transaction id %s', 'wpas-edd-payu-latam'), $txnid));
                } else if ($transactionState == 7) {
                    // Pending Payment
                    edd_update_payment_status( $payment_id, 'pending' );
                    edd_insert_payment_note( $payment_id, sprintf(__('Payment done via PayUmoney Latam with transaction id %s', 'wpas-edd-payu-latam'), $txnid));
                } else {
                    // Error
                    edd_update_payment_status( $payment_id, 'failed' );
                    edd_insert_payment_note( $payment_id, sprintf(__('Payment done via PayUmoney Latam with transaction id %s', 'wpas-edd-payu-latam'), $txnid));
                }
            }

        }
    }

}